
from pyvboxmanage import __title__ as NAME
from pyvboxmanage import __version__ as VERSION
from pyvboxmanage.cli import click
from pyvboxmanage.exceptions.PyVBoxManageException import PyVBoxManageException


def pyvboxmanage():

    try:
        click.pyvboxmanage()
    except PyVBoxManageException as e:
        print('')
        print('{} v{}'.format(NAME, VERSION))
        print('ERROR: ', end='')
        for err in iter(e.args):
            print(err)
        print('')
        exit(1)





import logging
import platform
from pyvboxmanage.utils.exec_command import exec_command
from pyvboxmanage.exceptions.PyVBoxManageException import PyVBoxManageException


logger = logging.getLogger(__name__)


def vboxmanage_binary(dry_run=False):

    if 'Windows' in platform.system():
        bin_file = 'VBoxManage'
    else:
        bin_file = 'vboxmanage'

    if dry_run:
        logger.debug('Dry mode VBoxManage binary set to {}'.format(bin_file))
        return bin_file

    stdout, stderr, returncode = exec_command(bin_file)

    if int(returncode) > 0:
        raise PyVBoxManageException('Unable to locate {} in system PATH'.format(bin_file))

    if stderr:
        raise PyVBoxManageException('Unexpected STDERR output from {}'.format(bin_file), stderr)

    logger.debug('VBoxManage binary available as {}'.format(bin_file))
    return bin_file




import time
import logging

TIMESTAMP_FORMAT = '%Y-%m-%d %H:%M:%S %Z%z'


class LoggerException(Exception):
    pass


class LoggerColoredFormatter(logging.Formatter):

    color_line = '\x1b[90m'  # grey
    color_reset = '\x1b[0m'  # reset

    def __init__(self, **kwargs):
        if 'fmt' in kwargs:
            kwargs['fmt'] = '{}{}{}'.format(self.color_line, kwargs['fmt'], self.color_reset)
        logging.Formatter.__init__(self, **kwargs)

    def format(self, record):

        levelname = record.levelname.upper()

        if levelname == 'CRITICAL':
            color_code = '\x1b[41m'  # white-on-red
        elif levelname == 'ERROR':
            color_code = '\x1b[31m'  # red
        elif levelname in ('WARNING', 'WARN'):
            color_code = '\x1b[33m'  # yellow
        elif levelname == 'INFO':
            color_code = '\x1b[36m'  # cyan
        elif levelname == 'DEBUG':
            color_code = '\x1b[37m'  # white
        else:
            color_code = '\x1b[90m'  # grey

        record.levelname = '{}{}{}'.format(color_code, levelname, self.color_line)

        return logging.Formatter.format(self, record)


class Logger:

    name = None
    logger = None

    def __init__(self, name, level=None):

        if level is not None:
            log_level = level
        else:
            log_level = 'info'

        logger_init = logging.getLogger(name)
        if logger_init.level > 0:
            self.logger = logger_init
            return

        logger_init.setLevel(logging.DEBUG)

        log_level = log_level.upper()
        stream_handler = logging.StreamHandler()

        if log_level in ('CRITICAL', 'FATAL'):
            stream_handler.setLevel(logging.CRITICAL)
        elif log_level == 'ERROR':
            stream_handler.setLevel(logging.ERROR)
        elif log_level in ('WARNING', 'WARN'):
            stream_handler.setLevel(logging.WARNING)
        elif log_level == 'INFO':
            stream_handler.setLevel(logging.INFO)
        elif log_level == 'DEBUG':
            stream_handler.setLevel(logging.DEBUG)
        elif log_level is not None:
            raise LoggerException('unknown loglevel value', log_level)
        else:
            stream_handler.setLevel(logging.NOTSET)

        formatter = LoggerColoredFormatter(
            fmt='%(asctime)s - %(levelname)s - %(message)s',
            datefmt=TIMESTAMP_FORMAT
        )
        logging.Formatter.converter = time.localtime

        stream_handler.setFormatter(formatter)
        logger_init.addHandler(stream_handler)

        self.logger = logger_init


def init(name, level='info'):
    global __logger
    __logger = Logger(name, level=level).logger

def debug(message):
    __logger.debug(message)

def info(message):
    __logger.info(message)

def warning(message):
    __logger.warning(message)

def error(message):
    __logger.error(message)

def critical(message):
    __logger.critical(message)



def dict_merge(a, b, path=None):
    if path is None:
        path = []
    for key in b.keys():
        if key in a.keys() and isinstance(a[key], dict) and isinstance(b[key], dict):
            dict_merge(a[key], b[key], path + [str(key)])
        else:
            a[key] = b[key]
    return a




import os
import yaml
import logging
from pyvboxmanage.utils.dict_merge import dict_merge
from pyvboxmanage.exceptions.PyVBoxManageException import PyVBoxManageException
from pyvboxmanage import __config_import_recursion_limit__


logger = logging.getLogger(__name__)


def load_configuration_files(configuration_files):

    config = {}
    for configuration_file in configuration_files:
        config = dict_merge(config, load_configuration_file(configuration_file))

    config = replace_config_vars(config)
    return normalize_config(config)


def load_configuration_file(configuration_file, __level=0):

    if not os.path.exists(configuration_file):
        raise PyVBoxManageException('Configuration file not found', configuration_file)

    if __level >= __config_import_recursion_limit__:
        raise PyVBoxManageException('Configuration recursion limit reached, likely circular import references', __level)

    with open(configuration_file) as f:
        content = f.read()

    config = parse_yaml_content(content)
    logger.debug('Loaded configuration file {}'.format(configuration_file))

    if 'imports' in config.keys() and type(config['imports']) is list:
        for configuration_import_file in config['imports']:
            if not os.path.exists(configuration_import_file):
                configuration_import_file = os.path.join(os.path.dirname(configuration_file), configuration_import_file)
            config = dict_merge(config, load_configuration_file(configuration_import_file, __level+1))

    return config


def replace_config_vars(config):

    content = yaml.dump(config)

    if 'vars' in config.keys():
        if type(config['vars']) is not dict:
            raise PyVBoxManageException('Unexpected "vars" configuration', config['vars'])

        format_kwargs = {}
        for key, value in config['vars'].items():
            tag = '{' + key + '}'
            if tag in content:
                format_kwargs[key] = value

        try:
            content = content.format(**format_kwargs)
        except KeyError as e:
            raise PyVBoxManageException('Missing variable setting in configuration, is it set?', e)

        config = parse_yaml_content(content)

    return config


def parse_yaml_content(content):
    try:
        config = yaml.safe_load(content)
    except yaml.parser.ParserError as e:
        raise PyVBoxManageException('Configuration file format error', e)

    if type(config) is not dict:
        raise PyVBoxManageException('Unexpected configuration file format')

    return config


def normalize_config(config):

    if 'pyvboxmanage' not in config.keys():
        raise PyVBoxManageException('Unexpected configuration file format, missing "pyvboxmanage" root')

    if type(config['pyvboxmanage']) is not list:
        raise PyVBoxManageException('Unexpected configuration file format, "pyvboxmanage" content is not a list')

    items = []

    for item in config['pyvboxmanage']:
        if type(item) is not dict:
            raise PyVBoxManageException('Unexpected configuration file format, item is not a dict', item)

        command = options = arguments = outputs = triggers = returncodes = timeout = None

        for key, value in item.items():

            # arguments
            if key.lower() in ['arg', 'argument', 'args', 'arguments']:
                arguments = __normalize_arguments(key, value)

            # options
            elif key.lower() in ['opts', 'options']:
                options = __normalize_options(key, value)

            # outputs
            elif key.lower() in ['out', 'output', 'outputs']:
                outputs = __normalize_outputs(key, value)

            # returncodes
            elif key.lower() in ['returncode', 'return_code', 'returncodes', 'return_codes']:
                returncodes = __normalize_returncodes(key, value)

            # triggers
            elif key.lower() in ['trigger', 'triggers']:
                triggers = __normalize_triggers(key, value)

            # timeout
            elif key.lower() in ['timeout']:
                timeout = __normalize_timeout(key, value)

            # command
            else:
                if command is not None:
                    raise PyVBoxManageException('Unexpected configuration attribute "{}" in "{}"'.format(key, command))
                command = key

        items.append({
            'command': command,
            'options': options,
            'arguments': arguments,
            'outputs': outputs,
            'triggers': triggers,
            'returncodes': returncodes,
            'timeout': timeout
        })

    return items


def __normalize_arguments(key, value):
    if type(value) is str:
        return [value]
    return value


def __normalize_options(key, value):
    if type(value) is not dict:
        raise PyVBoxManageException('Unexpected options configuration {}'.format(key), value)
    return value


def __normalize_outputs(key, value):
    if type(value) is str:
        return [value]
    return value


def __normalize_returncodes(key, value):
    if type(value) in [str, int]:
        returncodes = [value]
    elif type(value) is list:
        returncodes = value
    else:
        raise PyVBoxManageException('Unexpected returncodes type', value)

    for returncode_index, returncode in enumerate(returncodes):
        if type(returncode) is str:
            try:
                returncodes[returncode_index] = int(returncode)
            except ValueError as e:
                raise PyVBoxManageException(e, value)
        elif type(returncode) is not int:
            raise PyVBoxManageException('Unexpected returncodes value type', value)

    return returncodes


def __normalize_triggers(key, value):
    if type(value) is dict:
        triggers = [value]
    elif type(value) is list:
        triggers = value
    else:
        raise PyVBoxManageException('Unexpected triggers configuration {}'.format(key), value)

    for trigger_index, trigger in enumerate(triggers):
        # source
        if 'source' not in trigger.keys():
            triggers[trigger_index]['source'] = 'stdout'
        if trigger['source'].lower() not in ['stdout', 'stderr', 'returncode']:
            raise PyVBoxManageException('Unexpected trigger source supplied', trigger)
        triggers[trigger_index]['source'] = triggers[trigger_index]['source'].lower()

        # string
        if 'string' not in trigger.keys():
            raise PyVBoxManageException('Required "string" trigger setting missing', trigger)

        # condition
        if 'condition' not in trigger.keys():
            triggers[trigger_index]['condition'] = 'present'
        elif trigger['condition'].lower() in ['present']:
            triggers[trigger_index]['condition'] = 'present'
        elif trigger['condition'].lower() in ['not present', 'not_present', 'notpresent', '!present', '! present']:
            triggers[trigger_index]['condition'] = 'not_present'
        else:
            raise PyVBoxManageException('Unexpected trigger condition supplied', trigger)

    return triggers


def __normalize_timeout(key, value):
    if type(value) is str:
        try:
            timeout = int(value)
        except ValueError as e:
            raise PyVBoxManageException(e, value)
    elif type(value) is int:
        timeout = value
    else:
        raise PyVBoxManageException('Unexpected triggers configuration {}'.format(key), value)

    return timeout




import click

from pyvboxmanage import __title__ as NAME
from pyvboxmanage import __version__ as VERSION
from pyvboxmanage.utils import logger
from pyvboxmanage.PyVBoxManage import PyVBoxManage


@click.command(no_args_is_help=True)
@click.argument('configuration_files', required=True, nargs=-1)
@click.option('-v', '--verbose', is_flag=True, help='Verbose logging messages (debug level).')
@click.option('-q', '--quiet', is_flag=True, help='Quiet mode, with priority over --verbose')
@click.option('-d', '--dry-run', is_flag=True, help='Dry run mode, output the commands that would execute only.')
@click.version_option(VERSION)
def pyvboxmanage(configuration_files, verbose, quiet, dry_run):
    """
    PyVBoxManage is a wrapper tool around VBoxManage that facilitates the orchestration of VBoxManage commands from a
    simple YAML configuration file that matches the input opts/args for VBoxManage.  This makes it possible to
    implement common sequences of VBoxManage commands such as spinning up a new dev/test instance with different
    hardware just by using a single command line with a configuration file.

    Variables, output redirection, exit-triggers and returncode-exceptions are available to make flexible setups.

    Documentation available https://pyvboxmanage.readthedocs.io
    """

    ctx = click.get_current_context()
    ctx.ensure_object(dict)

    if quiet:
        logger.init(name=NAME, level='critical')
    elif verbose:
        logger.init(name=NAME, level='debug')
    else:
        logger.init(name=NAME, level='info')

    logger.debug('{} v{}'.format(NAME, VERSION))

    PyVBoxManage(configuration_files=configuration_files, dry_run=dry_run).main()




import sys
import logging
from pyvboxmanage import __exec_timeout_default__ as EXEC_TIMEOUT_DEFAULT
from pyvboxmanage.exceptions.PyVBoxManageException import PyVBoxManageException
from pyvboxmanage.utils.config import load_configuration_files
from pyvboxmanage.utils.bin_file import vboxmanage_binary
from pyvboxmanage.utils.exec_command import exec_command


logger = logging.getLogger(__name__)


class PyVBoxManage:

    config = None
    dry_run = None
    bin_file = None

    def __init__(self, configuration_files, dry_run=False):
        self.dry_run = dry_run
        self.config = load_configuration_files(configuration_files)
        self.bin_file = vboxmanage_binary(dry_run=dry_run)

    def main(self):
        for item in self.config:
            stdout, stderr, returncode = self.config_item_runner(item)
            self.config_item_outputs(stdout, item['outputs'])
            if self.config_item_triggers(stdout, stderr, returncode, item['triggers']) is False:
                logger.info('Stopping "{}" because trigger condition does not match'.format(item['command']))
                return

    def config_item_triggers(self, stdout, stderr, returncode, setup):
        if setup is None:
            setup = []
        elif type(setup) is not list:
            raise PyVBoxManageException('Unexpected triggers config', setup)

        for trigger_setup in setup:
            if trigger_setup['source'] == 'stdout':
                content = stdout
            elif trigger_setup['source'] == 'stderr':
                content = stderr
            elif trigger_setup['source'] == 'returncode':
                content = returncode
            else:
                raise PyVBoxManageException('Unexpected trigger source condition')

            if trigger_setup['string'] in content.decode('utf8'):
                logger.debug('Trigger string "{}" is present'.format(trigger_setup['string']))
                if trigger_setup['condition'] != 'present':
                    logger.debug('Trigger string does not match expected "present" condition state')
                    return False
            else:
                logger.debug('Trigger string "{}" is not present'.format(trigger_setup['string']))
                if trigger_setup['condition'] != 'not_present':
                    logger.debug('Trigger string does not match expected "not_present" condition state')
                    return False
        return True

    def config_item_outputs(self, stdout, setup):
        if setup is None:
            setup = []
        elif type(setup) is not list:
            raise PyVBoxManageException('Unexpected outputs config', setup)

        for output_target in setup:
            if output_target.lower() in ['/dev/null', 'devnull', 'null']:
                continue
            elif output_target.lower() in ['stdout']:
                print(stdout.decode('utf8'), file=sys.stdout)
            elif output_target.lower() in ['stderr']:
                print(stdout.decode('utf8'), file=sys.stderr)
            else:
                try:
                    with open(output_target, mode='wb') as f:
                        f.write(stdout)
                except Exception as e:
                    raise PyVBoxManageException('Unable to write to {}'.format(output_target), e)
                logger.debug('Output written to {}'.format(output_target))

    def config_item_runner(self, item):
        command_line = self.render_command(item)
        logger.debug('Rendered command line: {}'.format(command_line))
        if item['timeout']:
            exec_timeout = item['timeout']
        else:
            exec_timeout = EXEC_TIMEOUT_DEFAULT
        logger.debug('Command line exec timeout {}'.format(exec_timeout))

        if self.dry_run is True:
            stdout, stderr, returncode = b'', b'', 0
        else:
            stdout, stderr, returncode = exec_command(command_line, timeout=exec_timeout)

        if item['returncodes']:
            if returncode in item['returncodes']:
                if returncode > 0:
                    logger.warning('Command line "{}" returncode "{}" configured as okay'.format(command_line, returncode))
                else:
                    logger.info('Successfully executed command line "{}"'.format(command_line))
                return stdout, stderr, returncode
            else:
                raise PyVBoxManageException('Unexpected "{}" returncode from command execution'.format(returncode), command_line)

        if returncode > 0:
            raise PyVBoxManageException('Unexpected response from {}'.format(command_line), stderr)

        logger.info('Successfully executed command line "{}"'.format(command_line))
        return stdout, stderr, returncode

    def render_command(self, command):
        command_line = '{} {}'.format(self.bin_file, command['command'])

        if type(command['arguments']) is list:
            for item_argument in command['arguments']:
                if ' ' in item_argument:
                    command_line += ' "{}"'.format(item_argument)
                else:
                    command_line += ' {}'.format(item_argument)

        if type(command['options']) is dict:
            for option_key, option_value in command['options'].items():
                if type(option_value) is bool and option_value is True:
                    command_line += ' --{}'.format(option_key)
                else:
                    command_line += ' --{} "{}"'.format(option_key, option_value)

        return command_line



# This file was auto-generated by Shut. DO NOT EDIT
# For more information about Shut, check out https://pypi.org/project/shut/

from __future__ import print_function
import io
import os
import setuptools
import sys

readme_file = 'README.md'
if os.path.isfile(readme_file):
  with io.open(readme_file, encoding='utf8') as fp:
    long_description = fp.read()
else:
  print("warning: file \"{}\" does not exist.".format(readme_file), file=sys.stderr)
  long_description = None

requirements = [
  'click',
  'pyyaml',
]

setuptools.setup(
  name = 'pyvboxmanage',
  version = '0.3.0',
  author = 'Nicholas de Jong',
  author_email = 'me@nicholasdejong.com',
  description = 'Wrapper tool around VBoxManage that facilitates the orchestration of VBoxManage commands from a simple YAML configuration file that matches the input opts/args for VBoxManage.',
  long_description = long_description,
  long_description_content_type = 'text/markdown',
  url = 'https://pyvboxmanage.readthedocs.io',
  license = 'BSD2',
  packages = setuptools.find_packages('src', ['test', 'test.*', 'tests', 'tests.*', 'docs', 'docs.*']),
  package_dir = {'': 'src'},
  include_package_data = True,
  install_requires = requirements,
  extras_require = {},
  tests_require = [],
  python_requires = '>=3.5.0,<4.0.0',
  data_files = [],
  entry_points = {
    'console_scripts': [
      'pyvboxmanage = pyvboxmanage.cli.entrypoints:pyvboxmanage',
    ]
  },
  cmdclass = {},
  keywords = ['virtuabox', 'vbox', 'vboxmanage', 'devops'],
  classifiers = ['Environment :: Console', 'Intended Audience :: Developers', 'Intended Audience :: Information Technology', 'Programming Language :: Python :: 3.5', 'Programming Language :: Python :: 3.6', 'Programming Language :: Python :: 3.7', 'Programming Language :: Python :: 3.8', 'Programming Language :: Python :: 3.9', 'License :: OSI Approved :: BSD License'],
  zip_safe = True,
)




import subprocess
from pyvboxmanage.exceptions.PyVBoxManageException import PyVBoxManageException


def exec_command(command_line, timeout=10):
    try:
        sp = subprocess.run(command_line, shell=True, capture_output=True, timeout=timeout)
    except subprocess.TimeoutExpired as e:
        raise PyVBoxManageException(e)

    return sp.stdout, sp.stderr, sp.returncode
